<?php
//
// Linux Server Config Wizard
// Copyright (c) 2010-2018 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

s3_configwizard_init();

function s3_configwizard_init()
{
    $name = "s3";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.0.1",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Monitor an Amazon S3 Instance."),
        CONFIGWIZARD_DISPLAYTITLE => "Amazon S3",
        CONFIGWIZARD_FUNCTION => "s3_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "s3.png",
        CONFIGWIZARD_FILTER_GROUPS => array('amazon'),
        CONFIGWIZARD_REQUIRES_VERSION => 500
    );
    register_configwizard($name, $args);
}

/**
 * @return int
 *          0 = good to go
 *          1 = prereqs non-existent
 *          2 = may need to upgrade boto3
 */

function s3_configwizard_check_prereqs()
{
    // Plugin doesn't exist
    if (!file_exists("/usr/local/nagios/libexec/check_s3.py") && !is_dev_mode()) {
        return 4; // plugin non-existent
    }

    $preferred_version = '1.4.7';

    $found_pip = false;
    $pip_output = array();

    // See if boto3 is installed via pip and get the version number
    // $pip_command = "pip show boto3"; <-- THIS HAS LOW PERFORMANCE SOMETIMES, DO NOT USE
    $pip_command = 'python -c "import boto3"';
    exec($pip_command, $pip_output, $pip_return);

    // If neither yum nor pip returned anything, there is no need to continue
    if ($pip_return !== 0) {
        return 1; // prereqs non-existent
    }
}

/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */

function s3_configwizard_func($mode = "", $inargs = null, &$outargs, &$result)
{

    $wizard_name = "s3";
    $local_url = get_base_url();

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {

        case CONFIGWIZARD_MODE_GETSTAGE1HTML:
            
            $check_prereqs = s3_configwizard_check_prereqs();

            // Remove any existing session variables if this is the first time on the page
            if (isset($_POST['backButton']) == false) {
                unset($_SESSION['bucketlist']);
                unset($_SESSION['checkboxlist']);
            }
            if (isset($_SESSION['metriclist'])) {
                unset($_SESSION['metriclist']);
            }


            if ($check_prereqs == 1) {
                $output .= '<div class="message errorMessage" style="padding: 10px; margin-bottom: 20px;"><p><b>' . _('Error') . ':</b> ' . _('It looks like you are missing boto3 on your Nagios XI server.') . '</p><p>' . _('To use this wizard you must install boto3 on your server. If you are using CentOS or RHEL you can run:') . '</p><pre>pip install boto3</pre></div>';

            } elseif ($check_prereqs == 4) {
                $output .= '<div class="message errorMessage" style="padding: 10px; margin-bottom: 20px;"><p><b>' . _('Error') . ':</b> ' . _('It looks like you are missing the check_s3.py plugin on your Nagios XI server.</p></div>');
            } else {

            $address = grab_array_var($inargs, "address", "");
            $address = nagiosccm_replace_user_macros($address);

            if (isset($_SESSION['bucketlist'])) {
                $bucketlist = $_SESSION['bucketlist'];
            } else {
                $bucketlist = grab_array_var($inargs, "bucketlist", "");
            }

            if (isset($_SESSION['checkboxlist'])) {
                $bucketlist = $_SESSION['checkboxlist'];
            } else {
                $bucketlist = grab_array_var($inargs, "checkboxlist", "");
            }

            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");

            if (isset($_SESSION['checkboxlist'])) {
                $checkboxlist = $_SESSION['checkboxlist'];
            } else {
                $checkboxlist = grab_array_var($inargs, "checkboxlist", "");
            }

            if ($credsfilepath == "") {
                $credsfilepath = "/usr/local/nagiosxi/etc/.aws/credentials";
            }

            if ($configfilepath == "") {
                $configfilepath = "/usr/local/nagiosxi/etc/.aws/config";
            }

            $linuxdistro = grab_array_var($inargs, "linuxdistro", "");

            $output = '
            <h5 class="ul">'._('AWS Account Information').'</h5>
            <table class="table table-condensed table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('Access Key ID') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskeyid" id="accesskeyid" value="' . encode_form_val($accesskeyid) . '" class="textfield form-control credinput" ' . (checkbox_binary($staticcreds) ? "disabled" : "") . ' autocomplete="off">
                        <div class="subtext">' . _('The Access Key ID of the bucket to be monitored') . '.</div>
                    </td>
                </tr>
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('Secret Access Key') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskey" id="accesskey" value="' . encode_form_val($accesskey) . '" class="textfield form-control credinput" ' . (checkbox_binary($staticcreds) ? "disabled" : "") . ' autocomplete="off">
                        <div class="subtext">' . _('The Secret Access Key of the bucket to be monitored') . '.</div>
                    </td>
                </tr>
            </table>

            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td style="width: 135px;"></td>
                    <td>
                        <input type="checkbox" class="checkbox" id="staticcreds" name="staticcreds" ' . is_checked(checkbox_binary($staticcreds), "1") . '>
                    </td>
                    <td>
                        <label class="normal" for="staticcreds">
                            <b>' . _('Static credentials file') . '</b><br>
                            ' . _('Use a statically defined AWS credentials file') . '.
                        </label>

                        <i class="credtooltip fa fa-question-circle fa-14" data-placement="right" data-content="If this box is checked, Nagios will read a local file containing your AWS credentials that follows Amazon\'s syntax.  Read more about credentials files <a target=\'' . '_blank' . '\' href=\'' . 'http://docs.aws.amazon.com/cli/latest/userguide/cli-config-files.html' . '\'>here</a>.">
                    </td>
                </tr>
            </table>

            <table class="table table-condensed table-no-border table-auto-width" id="credsfiletable">
                <tr>
                    <td class="vt" id="filepath" style="width: 135px;">
                        <label>' . _('File path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="credsfilepath" id="credsfilepath" value="' . encode_form_val($credsfilepath) . '" class="textfield form-control" disabled>
                        <div class="subtext">' . _('The absolute file path to the Amazon Credentials file (<b>Default:</b> /usr/local/nagiosxi/etc/.aws/credentials)') . '.</div>
                        <div class="subtext">' . _('<b>Note: Ensure the nagios group has permissions for the entirety of the file path leading to the credentials file, including the file itself</b>') . '.</div>
                    </td>
                </tr>
            </table>

            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td style="width: 135px;"></td>
                    <td>
                        <input type="checkbox" class="checkbox" id="staticconfig" name="staticconfig" ' . is_checked(checkbox_binary($staticconfig), "1") . '>
                    </td>
                    <td>
                        <label class="normal" for="staticconfig">
                            <b>' . _('Static configuration file') . '</b><br>
                            ' . _('Use a statically defined AWS configurations file') . '.
                        </label>
                        <i class="credtooltip fa fa-question-circle fa-14" data-placement="right" data-content="If this box is checked, Nagios will read a local file containing your AWS configuration that follows Amazon\'s syntax.  Read more about configuration files <a target=\'' . '_blank' . '\' href=\'' . 'http://docs.aws.amazon.com/cli/latest/userguide/cli-config-files.html' . '\'>here</a>.">
                    </td>
                </tr>
            </table>

            <table class="table table-condensed table-no-border table-auto-width" id="configfiletable">
                <tr>
                    <td class="vt" style="width: 135px;">
                        <label>' . _('File path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="configfilepath" id="configfilepath" value="' . encode_form_val($configfilepath) . '" class="textfield form-control" disabled>
                        <div class="subtext">' . _('The absolute file path to the Amazon Configuration file (<b>Default:</b> /usr/local/nagiosxi/etc/.aws/config)') . '.</div>
                        <div class="subtext">' . _('<b>Note: Ensure the nagios group has permissions for the entirety of the file path leading to the config file, including the file itself</b>') . '.</div>
                    </td>
                </tr>
            </table>

            <h5 class="ul">'._('S3 Bucket Information').'</h5>
            <p>' . _('Select the buckets you would like to monitor, then enter the region name by the selected buckets.') . '</p>';

            if ($bucketlist == '') {
                $output .= '
                <div class="alert alert-info loader" id="bucket-notice" style="max-width: 500px; text-align: center; height: 117px;">
                    <div id="ajax-loader" class="sk-spinner sk-spinner-rotating-plane" style="background-color: #4D89F9; display: none; margin-top: 27px;">
                    </div>
                    <div id="info-placeholder">
                        <p class="center available-buckets" id="no-buckets" style="padding-top: 5px; padding-bottom: 15px;">We don\'t know which buckets are available yet! Fill out the AWS Account Information section, then click "Get Available Buckets".
                        </p> 
                    </div>
                    <button type="button" id="get-buckets" class="btn btn-sm btn-default center available-buckets">Get Available Buckets</button>
               
                </div>
                <table class="table table-auto-width table-no-border" id="bucket-container">
                </table>';
            } else {
                $output .= '
                <table class="table table-auto-width table-no-border" id="bucket-container">';

                //$bucketlist = json_decode($_SESSION['bucketlist']);

                $counter = 0;
                foreach ($bucketlist as $bucket => $region) {
                    $output .= '<tr><td class="vt"><input type="checkbox" class="bucket-list-checkbox" name="checkboxlist[' . $counter . ']" style="margin-top: 2px;"' . (checkbox_binary($checkboxlist[$counter]) ? "checked" : "") . '></td><td><input type="text" class="form-control" name="bucketlist[' . $counter . ']" size="45" value="' . $bucket . '"></td><td><input type="text" class="form-control" name="regionlist[' . $counter . ']" value="' . $region . '"></td></tr>';
                    $counter++;
                }

                $output .= '</table>';
            }
            
            $output .='
            <script type="text/javascript">

                $("#get-buckets").on("click", function() {

                    $("#get-buckets").hide();
                    $("#no-buckets").hide();
                    $("#info-placeholder").empty();
                    $("#ajax-loader").show();

                    var accesskeyid = $("#accesskeyid").val();
                    var accesskey = $("#accesskey").val();

                    // Base64 encode the strings so we can pass them as a query string
                    accesskeyid = btoa(accesskeyid);
                    accesskey = btoa(accesskey);

                    var formvalues = "mode=getbuckets";

                    if ($("#staticcreds").is(":checked")) {
                        var staticcreds = $("#staticcreds").val();
                        var credsfilepath = $("#credsfilepath").val();
                        formvalues += "&staticcreds=" + staticcreds;
                        formvalues += "&credsfilepath=" + credsfilepath;
                    }

                    formvalues += "&accesskeyid=" + accesskeyid;
                    formvalues += "&accesskey=" + accesskey;

                    full_url = base_url + "includes/configwizards/amazon_s3/s3.ajax.php?" + formvalues;

                    $.ajax({
                        url: full_url,
                        dataType: "json",
                        success: function(data) {
                            $("#bucket-notice").hide();

                            $("#bucket-container").append("<thead><tr><th><input type=\'checkbox\' id=\'select-all-buckets\' checked></th><th>Bucket Name</th><th>Bucket Region</th></tr></thead>");

                            var counter = 0;

                            $.each(data, function( index, value ) {
                                $("#bucket-container").append(\'<tr><td class="vt"><input type="checkbox" class="bucket-list-checkbox" name="checkboxlist[\' + counter + \']" style="margin-top: 2px;" checked></td><td><input type="text" class="form-control" name="bucketlist[\' + counter + \']" size="45" value="\' + index + \'"></td><td><input type="text" class="form-control" name="regionlist[\' + counter + \']" value="\' + value + \'"></td></tr>\');
                                counter++;
                            });
                        },
                        error: function(xmlhttprequest, textstatus, message) {
                            if (textstatus === "parsererror") {
                                $("#ajax-loader").hide();
                                $("#info-placeholder").html(\'<p style="margin-bottom: 15px; padding-top: 5px;">There were no buckets found. Verify your AWS credentials and try again.</p>\');
                                $("#get-buckets").show();
                            }
                        }
                    });
                });

                $("#bucket-container").on("click", "thead > tr > th > input:checkbox", function() {
                    if ($(this).prop("checked") ==  true) {
                        $(".bucket-list-checkbox").prop("checked", true);
                    } else {
                        $(".bucket-list-checkbox").prop("checked", false);
                    }
                });

                $(function(){
                    $(".credtooltip").popover({ html: true });
                });

                document.addEventListener("DOMContentLoaded", function() {
                    var credtable = document.getElementById("credsfiletable");
                    var credinputs = document.getElementsByClassName("credinput");

                    document.querySelector("#staticcreds").addEventListener("change", function() {

                        for (var i = 0; i < credinputs.length; i++) {
                            if (credinputs[i].disabled === true) {
                                credinputs[i].disabled = false;
                            } else {
                                credinputs[i].disabled = true;
                            }
                        }
                    
                        if (document.getElementById("credsfilepath").disabled === true) {
                            document.getElementById("credsfilepath").disabled = false;
                        } else {
                            document.getElementById("credsfilepath").disabled = true;
                        }
                    });
                });

            </script>

            

            <script type="text/javascript">

            $(function(){
                $(".configtooltip").popover({ html: true });
            });

            document.addEventListener("DOMContentLoaded", function() {
                var configtable = document.getElementById("configfiletable");
                var configinputs = document.getElementsByClassName("configinputs");

                document.querySelector("#staticconfig").addEventListener("change", function() {

                    for (var i = 0; i < configinputs.length; i++) {
                        if (configinputs[i].disabled === true) {
                            configinputs[i].disabled = false;
                        } else {
                            configinputs[i].disabled = true;
                        }
                    }

                    if (document.getElementById("configfilepath").disabled === true) {
                        document.getElementById("configfilepath").disabled = false;
                    } else {
                        document.getElementById("configfilepath").disabled = true;
                    }

                });
            });

            </script>';
            }
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // Get variables that were passed to us
            
            if ($bucketlist == "") {
                $tempbucketlist = (array) $_SESSION['bucketlist'];

                $counter = 0;
                foreach ($tempbucketlist as $bucket => $region) {
                    $bucketlist[$counter] = $bucket;
                    $counter++;
                }
            } else {
                $bucketlist = grab_array_var($inargs, "bucketlist", "");
            }

            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $regionlist = grab_array_var($inargs, "regionlist", "");
            $address = grab_array_var($inargs, "address", "");

            $outargs[CONFIGWIZARD_PASSBACK_DATA]['checkboxlist'] = $checkboxlist;

            if ($_SESSION['checkboxlist'] != NULL) {
                $checkboxlist = $_SESSION['checkboxlist'];
            } else {
                $checkboxlist = grab_array_var($inargs, "checkboxlist", "");
            }

            $_SESSION['checkboxlist'] = $checkboxlist;

            $staticcreds = grab_array_var($inargs, "staticcreds", "off");
            $staticconfig = grab_array_var($inargs, "staticconfig", "off");

            // Take each bucket name and get available metrics, store them in an array
            foreach($bucketlist as $key=>$value) {
                if (!$checkboxlist[$key]) {
                    continue;
                } else {

                    $cmd = "/usr/local/nagios/libexec/check_s3.py --changemode 'getmetrics' --accesskeyid '" . $accesskeyid . "' --secretaccesskey '" . $accesskey . "' --bucketname '" . $value . "' --region '" . $regionlist[$key] . "'";
                    $cmd = escapeshellcmd($cmd);

                    $returnedmetrics = shell_exec($cmd);
                    $decodedmetrics = json_decode($returnedmetrics, true);

                    $mergedarray = array();

                    foreach($decodedmetrics as $i => $v) {

                        $mergedarray[$i] = $v;                        
                        $mergedarray['Region'] = $regionlist[$key];
                    }

                    $metriclist[$value] = $mergedarray;
                
                }
            }

            $outargs[CONFIGWIZARD_PASSBACK_DATA]["metriclist"] = $metriclist;

            // Check for errors
            $errors = 0;
            $errmsg = array();

            if (empty($accesskeyid) and $staticcreds == "off") {
                $errmsg[$errors++] = "Specify either an Access Key ID or a static credentials file.";
            }

            if (empty($accesskey) and $staticcreds == "off") {
                $errmsg[$errors++] = "Specify either a Secret Access Key or a static credentials file.";
            }

            if (empty($regionlist) and $staticconfig == "off") {
                $errmsg[$errors++] = "Specify either a region or a static configuration file.";
            }

            if (empty($bucketlist) && empty($checkboxlist)) {
                $errmsg[$errors++] = "No buckets selected.";
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // Get variables that were passed to us

            $bucketlist = grab_array_var($inargs, "bucketlist", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $regionlist = grab_array_var($inargs, "regionlist", "");
            $address = grab_array_var($inargs, "address", "");
            $checkboxlist = grab_array_var($inargs, "checkboxlist", "");

            $outargs[CONFIGWIZARD_PASSBACK_DATA]['checkboxlist'] = $checkboxlist;

            if ($_SESSION['metriclist'] == NULL) {
                $metriclist = grab_array_var($inargs, "metriclist", "");
                $_SESSION['metriclist'] = $metriclist;
            } else {
                $metriclist = $_SESSION['metriclist'];
            }

            $ha = @gethostbyaddr($address);
            if (empty($ha)) {
                $ha = $address;
            }
            
            $hostnames = grab_array_var($inargs, "hostnames");

            $hostname = nagiosccm_replace_user_macros($hostname);

            $password = "";

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial", "");
            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));
            }
            if (!is_array($services)) {
                $services_default = array(
                    "ping" => 1,
                    "BucketSizeBytes" => 1,
                    "NumberOfObjects" => 1,
                    "AllRequests" => 0,
                    "GetRequests" => 0,
                    "PutRequests" => 0,
                    "DeleteRequests" => 0,
                    "HeadRequests" => 0,
                    "PostRequests" => 0,
                    "ListRequests" => 0,
                    "BytesDownloaded" => 0,
                    "BytesUploaded" => 0,
                    "4xxErrors" => 0,
                    "5xxErrors" => 0,
                    "FirstByteLatency" => 0,
                    "TotalRequestLatency" => 0

                );
                $services_default["servicestate"][0] = "on";
                $services_default["servicestate"][1] = "on";
                $services = grab_array_var($inargs, "services", $services_default);
            }

            //$serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");
            if ($serviceargs_serial != "") {
                $serviceargs_default = unserialize(base64_decode($serviceargs_serial));
            } else {
                $serviceargs_default = array(

                    "BucketSizeBytes_warning" => 1000000000,
                    "BucketSizeBytes_critical" => 1500000000,

                    "NumberOfObjects_warning" => 150,
                    "NumberOfObjects_critical" => 200,

                    "AllRequests_warning" => 1000,
                    "AllRequests_critical" => 1500,
                    
                    "GetRequests_warning" => 1000,
                    "GetRequests_critical" => 1500,
                    
                    "PutRequests_warning" => 1000,
                    "PutRequests_critical" => 1500,
                    
                    "DeleteRequests_warning" => 1000,
                    "DeleteRequests_critical" => 1500,
                    
                    "HeadRequests_warning" => 1000,
                    "HeadRequests_critical" => 1500,
                    
                    "PostRequests_warning" => 1000,
                    "PostRequests_critical" => 1500,
                    
                    "ListRequests_warning" => 1000,
                    "ListRequests_critical" => 1500,
                    
                    "BytesDownloaded_warning" => 500000000,
                    "BytesDownloaded_critical" => 750000000,
                    
                    "BytesUploaded_warning" => 500000000,
                    "BytesUploaded_critical" => 750000000,
                    
                    "4xxErrors_warning" => 25,
                    "4xxErrors_critical" => 50,
                    
                    "5xxErrors_warning" => 25,
                    "5xxErrors_critical" => 50,
                    
                    "FirstByteLatency_warning" => 150,
                    "FirstByteLatency_critical" => 300,
                    
                    "TotalRequestLatency_warning" => 150,
                    "TotalRequestLatency_critical" => 300
                );

                $serviceargs = grab_array_var($inargs, "serviceargs", $serviceargs_default);
            }

            if (!is_array($serviceargs)) {
                $serviceargs_default = array(

                    "BucketSizeBytes_warning" => 1000000000,
                    "BucketSizeBytes_critical" => 1500000000,

                    "NumberOfObjects_warning" => 150,
                    "NumberOfObjects_critical" => 200,

                    "AllRequests_warning" => 1000,
                    "AllRequests_critical" => 1500,
                    
                    "GetRequests_warning" => 1000,
                    "GetRequests_critical" => 1500,
                    
                    "PutRequests_warning" => 1000,
                    "PutRequests_critical" => 1500,
                    
                    "DeleteRequests_warning" => 1000,
                    "DeleteRequests_critical" => 1500,
                    
                    "HeadRequests_warning" => 1000,
                    "HeadRequests_critical" => 1500,
                    
                    "PostRequests_warning" => 1000,
                    "PostRequests_critical" => 1500,
                    
                    "ListRequests_warning" => 1000,
                    "ListRequests_critical" => 1500,
                    
                    "BytesDownloaded_warning" => 500000000,
                    "BytesDownloaded_critical" => 750000000,
                    
                    "BytesUploaded_warning" => 500000000,
                    "BytesUploaded_critical" => 750000000,
                    
                    "4xxErrors_warning" => 25,
                    "4xxErrors_critical" => 50,
                    
                    "5xxErrors_warning" => 25,
                    "5xxErrors_critical" => 50,
                    
                    "FirstByteLatency_warning" => 150,
                    "FirstByteLatency_critical" => 300,
                    
                    "TotalRequestLatency_warning" => 150,
                    "TotalRequestLatency_critical" => 300
                );

                $serviceargs = grab_array_var($inargs, "serviceargs", $serviceargs_default);
            }

            $output = '
            <input type="hidden" name="address" value="' . encode_form_val($address) . '">
            <input type="hidden" name="hostnames" value="' . encode_form_val($hostnames) . '">
            <input type="hidden" name="bucketlist" value="' . encode_form_val($bucketlist) . '">
            <input type="hidden" name="accesskeyid" value="' . encode_form_val($accesskeyid) . '">
            <input type="hidden" name="accesskey" value="' . encode_form_val($accesskey) . '">
            <input type="hidden" name="regionlist[]" value="' . encode_form_val($regionlist) . '">
            <input type="hidden" name="staticcreds" value="' . encode_form_val($staticcreds) . '">
            <input type="hidden" name="staticconfig" value="' . encode_form_val($staticconfig) . '">
            <input type="hidden" name="credsfilepath" value="' . encode_form_val($credsfilepath) . '">
            <input type="hidden" name="configfilepath" value="' . encode_form_val($configfilepath) . '">';
            
            $output .= '
            </table>

            <h5 class="ul">' . _('AWS Credentials') . '</h5>
            <table class="table table-condensed table-no-border table-auto-width">';

            if ($staticcreds == "on") {
                $output .= '
                <tr class="credentialsfilepath">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Credentials File Path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="credsfilepath" id="credsfilepath" value="' . encode_form_val($credsfilepath) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>';
            } else {
                $output .=
                '<tr class="specifiedcredentials">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Access Key ID') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskeyid" id="accesskeyid" value="' . encode_form_val($accesskeyid) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>
                <tr class="specifiedcredentials">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Secret Access Key') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="accesskey" id="accesskey" value="' . encode_form_val($accesskey) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>';
            }

            $output .= '
            </table>';

            if ($staticconfig == "on") {
                $output .= '
                <tr class="configurationfilepath">
                    <td class="vt" style="width: 170px;">
                        <label>' . _('Configuration File Path') . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="configfilepath" id="configfilepath" value="' . encode_form_val($configfilepath) . '" class="textfield form-control credsinput" disabled>
                    </td>
                </tr>';
            }

            foreach ($metriclist as $key => $value) {
                $output .= '<h5 class="ul">' . $key . ' (Region: ' . $value["Region"] . ')</h5>
                <table class="table table-no-border table-auto-width table-padded">
                <tr>
                    <td class="vt" style="width: 170px;">
                        <label for="serviceargs[' . $key . '][hostname]">' . _("Hostname") . ':</label>
                    </td>
                    <td>
                        <input type="text" size="40" name="serviceargs[' . $key . '][Hostname]" id="' . $key . '-hostname" value="' . encode_form_val($key) . '" class="textfield form-control">
                        <div class="subtext">'._("The hostname you would like associated with this host").'.</div>
                    </td>
                </tr>
                </table>

                <table class="table table-no-border table-auto-width table-padded">';

                // Sort the array alphabetically
                ksort($value);

                foreach ($value as $k => $v) {
                    switch($k) {
                        case'BucketSizeBytes':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-bucketsizebytes" name="serviceargs[' . $key . '][' . $k . '][BucketSizeBytes-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-bucketsizebytes" class="select-cf-option">'._('Bucket Size Bytes').'</label><br>
                                    ' . _('The amount of data in bytes stored in a bucket in the Standard storage class, Standard - Infrequent Access (Standard_IA) storage class, or the Reduced Redundancy Storage (RRS) class') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][BucketSizeBytes_warning]" value="' . $serviceargs_default["BucketSizeBytes_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][BucketSizeBytes_critical]" value="' . $serviceargs_default["BucketSizeBytes_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="BucketSizeBytes_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["StorageType"] . '" name="serviceargs['.$key.'][' . $k . '][BucketSizeBytes_storagetype]" id="BucketSizeBytes_storagetype"> &nbsp; <label for="BucketSizeBytes_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][BucketSizeBytes_filterid]" id="' . $key . '-BucketSizeBytes_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'NumberOfObjects':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-numberofobjects" name="serviceargs[' . $key . '][' . $k . '][NumberOfObjects-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-NumberOfObjects" class="select-cf-option">'._('Number of Objects').'</label><br>
                                    ' . _('The total number of objects stored in a bucket for all storage classes except for the GLACIER storage class') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][NumberOfObjects_warning]" value="' . $serviceargs_default["NumberOfObjects_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][NumberOfObjects_critical]" value="' . $serviceargs_default["NumberOfObjects_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="NumberOfObjects_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["StorageType"] . '" name="serviceargs['.$key.'][' . $k . '][NumberOfObjects_storagetype]" id="NumberOfObjects_storagetype"> &nbsp; <label for="NumberOfObjects_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][NumberOfObjects_filterid]" id="' . $key . '-NumberOfObjects_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'AllRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-allrequests" name="serviceargs[' . $key . '][' . $k . '][AllRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-AllRequests" class="select-cf-option">'._('All Requests').'</label><br>
                                    ' . _('The total number of HTTP requests made to an Amazon S3 bucket, regardless of type. If you’re using a metrics configuration with a filter, then this metric only returns the HTTP requests made to the objects in the bucket that meet the filter\'s requirements') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][AllRequests_warning]" value="' . $serviceargs_default["AllRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][AllRequests_critical]" value="' . $serviceargs_default["AllRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="AllRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][AllRequests_storagetype]" id="AllRequests_storagetype"> &nbsp; <label for="AllRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][AllRequests_filterid]" id="' . $key . '-AllRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'GetRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-getrequests" name="serviceargs[' . $key . '][' . $k . '][GetRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-GetRequests" class="select-cf-option">'._('GET Requests').'</label><br>
                                    ' . _('The number of HTTP GET requests made for objects in an Amazon S3 bucket. This doesn\'t include list operations') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][GetRequests_warning]" value="' . $serviceargs_default["GetRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][GetRequests_critical]" value="' . $serviceargs_default["GetRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="GetRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][GetRequests_storagetype]" id="GetRequests_storagetype"> &nbsp; <label for="GetRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][GetRequests_filterid]" id="' . $key . '-GetRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'PutRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-putrequests" name="serviceargs[' . $key . '][' . $k . '][PutRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-PutRequests" class="select-cf-option">'._('PUT Requests').'</label><br>
                                    ' . _('The number of HTTP PUT requests made for objects in an Amazon S3 bucket') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][PutRequests_warning]" value="' . $serviceargs_default["PutRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][PutRequests_critical]" value="' . $serviceargs_default["PutRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="PutRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][PutRequests_storagetype]" id="PutRequests_storagetype"> &nbsp; <label for="PutRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][PutRequests_filterid]" id="' . $key . '-PutRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'DeleteRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-deleterequests" name="serviceargs[' . $key . '][' . $k . '][DeleteRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-DeleteRequests" class="select-cf-option">'._('DELETE Requests').'</label><br>
                                    ' . _('The number of HTTP DELETE requests made for objects in an Amazon S3 bucket') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][DeleteRequests_warning]" value="' . $serviceargs_default["DeleteRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][DeleteRequests_critical]" value="' . $serviceargs_default["DeleteRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="DeleteRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][DeleteRequests_storagetype]" id="DeleteRequests_storagetype"> &nbsp; <label for="DeleteRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][DeleteRequests_filterid]" id="' . $key . '-DeleteRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'HeadRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-headrequests" name="serviceargs[' . $key . '][' . $k . '][HeadRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-HeadRequests" class="select-cf-option">'._('HEAD Requests').'</label><br>
                                    ' . _('The number of HTTP HEAD requests made to an Amazon S3 bucket') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][HeadRequests_warning]" value="' . $serviceargs_default["HeadRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][HeadRequests_critical]" value="' . $serviceargs_default["HeadRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="HeadRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][HeadRequests_storagetype]" id="HeadRequests_storagetype"> &nbsp; <label for="HeadRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][HeadRequests_filterid]" id="' . $key . '-HeadRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'PostRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-postrequests" name="serviceargs[' . $key . '][' . $k . '][PostRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-PostRequests" class="select-cf-option">'._('POST Requests').'</label><br>
                                    ' . _('The number of HTTP POST requests made to an Amazon S3 bucket') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][PostRequests_warning]" value="' . $serviceargs_default["PostRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][PostRequests_critical]" value="' . $serviceargs_default["PostRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="PostRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][PostRequests_storagetype]" id="PostRequests_storagetype"> &nbsp; <label for="PostRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][PostRequests_filterid]" id="' . $key . '-PostRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'ListRequests':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-listrequests" name="serviceargs[' . $key . '][' . $k . '][ListRequests-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-ListRequests" class="select-cf-option">'._('List Requests').'</label><br>
                                    ' . _('The number of HTTP requests that list the contents of a bucket') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][ListRequests_warning]" value="' . $serviceargs_default["ListRequests_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][ListRequests_critical]" value="' . $serviceargs_default["ListRequests_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="ListRequests_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][ListRequests_storagetype]" id="ListRequests_storagetype"> &nbsp; <label for="ListRequests_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][ListRequests_filterid]" id="' . $key . '-ListRequests_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'BytesDownloaded':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-bytesdownloaded" name="serviceargs[' . $key . '][' . $k . '][BytesDownloaded-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-BytesDownloaded" class="select-cf-option">'._('Bytes Downloaded').'</label><br>
                                    ' . _('The number bytes downloaded for requests made to an Amazon S3 bucket, where the response includes a body') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][BytesDownloaded_warning]" value="' . $serviceargs_default["BytesDownloaded_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][BytesDownloaded_critical]" value="' . $serviceargs_default["BytesDownloaded_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="BytesDownloaded_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][BytesDownloaded_storagetype]" id="BytesDownloaded_storagetype"> &nbsp; <label for="BytesDownloaded_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][BytesDownloaded_filterid]" id="' . $key . '-BytesDownloaded_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'BytesUploaded':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-bytesuploaded" name="serviceargs[' . $key . '][' . $k . '][BytesUploaded-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-BytesUploaded" class="select-cf-option">'._('Bytes Uploaded').'</label><br>
                                    ' . _('The number bytes uploaded that contain a request body, made to an Amazon S3 bucket') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][BytesUploaded_warning]" value="' . $serviceargs_default["BytesUploaded_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][BytesUploaded_critical]" value="' . $serviceargs_default["BytesUploaded_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="BytesUploaded_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][BytesUploaded_storagetype]" id="BytesUploaded_storagetype"> &nbsp; <label for="BytesUploaded_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][BytesUploaded_filterid]" id="' . $key . '-BytesUploaded_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'4xxErrors':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-4xxerrors" name="serviceargs[' . $key . '][' . $k . '][4xxErrors-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-4xxErrors" class="select-cf-option">'._('4xx Errors').'</label><br>
                                    ' . _('The number of HTTP 4xx client error status code requests made to an Amazon S3 bucket with a value of either 0 or 1. The average statistic shows the error rate, and the sum statistic shows the count of that type of error, during each period') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][4xxErrors_warning]" value="' . $serviceargs_default["4xxErrors_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][4xxErrors_critical]" value="' . $serviceargs_default["4xxErrors_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="4xxErrors_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][4xxErrors_storagetype]" id="4xxErrors_storagetype"> &nbsp; <label for="4xxErrors_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][4xxErrors_filterid]" id="' . $key . '-4xxErrors_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'5xxErrors':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-5xxerrors" name="serviceargs[' . $key . '][' . $k . '][5xxErrors-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-5xxErrors" class="select-cf-option">'._('5xx Errors').'</label><br>
                                    ' . _('The number of HTTP 5xx server error status code requests made to an Amazon S3 bucket with a value of either 0 or 1. The average statistic shows the error rate, and the sum statistic shows the count of that type of error, during each period') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][5xxErrors_warning]" value="' . $serviceargs_default["5xxErrors_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][5xxErrors_critical]" value="' . $serviceargs_default["5xxErrors_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="5xxErrors_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][5xxErrors_storagetype]" id="5xxErrors_storagetype"> &nbsp; <label for="5xxErrors_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][5xxErrors_filterid]" id="' . $key . '-5xxErrors_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'FirstByteLatency':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-firstbytelatency" name="serviceargs[' . $key . '][' . $k . '][FirstByteLatency-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-FirstByteLatency" class="select-cf-option">'._('First Byte Latency').'</label><br>
                                    ' . _('The per-request time from the complete request being received by an Amazon S3 bucket to when the response starts to be returned') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][FirstByteLatency_warning]" value="' . $serviceargs_default["FirstByteLatency_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][FirstByteLatency_critical]" value="' . $serviceargs_default["FirstByteLatency_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="FirstByteLatency_storagetype">Storage Type</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][FirstByteLatency_storagetype]" id="FirstByteLatency_storagetype"> &nbsp; <label for="FirstByteLatency_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][FirstByteLatency_filterid]" id="' . $key . '-FirstByteLatency_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'TotalRequestLatency':
                            $output .= '
                            <tr>
                                <td class="vt">
                                    <input type="checkbox" id="' . $key . '-totalrequestlatency" name="serviceargs[' . $key . '][' . $k . '][TotalRequestLatency-checkbox]" checked>
                                </td>
                                <td>
                                    <label for="' . $key . '-TotalRequestLatency" class="select-cf-option">'._('Total Request Latency').'</label><br>
                                    ' . _('The per-request time from the complete request being received by an Amazon S3 bucket to when the response starts to be returned') . '.
                                    <div class="pad-t5">
                                        <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][TotalRequestLatency_warning]" value="' . $serviceargs_default["TotalRequestLatency_warning"] . '" class="textfield form-control condensed"> &nbsp; <label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="9" name="serviceargs['.$key.'][' . $k . '][TotalRequestLatency_critical]" value="' . $serviceargs_default["TotalRequestLatency_critical"] . '" class="textfield form-control condensed"> &nbsp; <label for="TotalRequestLatency[]'.$key.'_storagetype">'._("Storage Type").'</label>&nbsp;<input type="text" class="form-control condensed" value="" name="serviceargs['.$key.'][' . $k . '][TotalRequestLatency_storagetype]" id="TotalRequestLatency_storagetype"> &nbsp; <label for="TotalRequestLatency_storagetype">'._("Filter ID").'</label>&nbsp;<input type="text" class="form-control condensed" value="' . $v["FilterId"] . '" name="serviceargs['.$key.'][' . $k . '][TotalRequestLatency_filterid]" id="' . $key . '-TotalRequestLatency_filterid">
                                    </div>
                                </td>
                            </tr>';
                            break;
                        case'Region':
                            $output .= '
                            <input type="hidden" name="serviceargs[' . $key . '][Region]" value="' . $value["Region"] . '">
                            ';
                            break;
                        default:
                            $output .= $k;
                            break;
                    }
                }
                $output .= '</table>';
            }

            $output .= '

            <div style="height: 20px;"></div>
            <script>
                $("#ToggleAllAdvanced").click(function() {
                    if ($(this).prop("checked") ==  true) {
                        $(".advanced-checkbox").prop("checked", true);
                    } else {
                        $(".advanced-checkbox").prop("checked", false);
                    }
                });
            </script>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostnames = grab_array_var($inargs, "hostnames");

            $bucketlist = grab_array_var($inargs, "bucketlist", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");

            $checkboxlist = grab_array_var($inargs, "checkboxlist", "");

            $outargs[CONFIGWIZARD_PASSBACK_DATA]['checkboxlist'] = $checkboxlist;

            $serviceargs = grab_array_var($inargs, "serviceargs", "");

            // Check for errors
            $errors = 0;
            $errmsg = array();

            foreach ($hostnames as $host) {

                if (is_valid_host_name($host) == false) {
                    $errmsg[$errors++] = "[" . $host . "] Is an invalid host name.";
                }
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostnames = grab_array_var($inargs, "hostnames");
            $bucketlist = grab_array_var($inargs, "bucketlist", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");

            $checkboxlist = grab_array_var($inargs, "checkboxlist", "");

            $outargs[CONFIGWIZARD_PASSBACK_DATA]['checkboxlist'] = $checkboxlist;

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));
            } else {
                $services = grab_array_var($inargs, "services");
            }

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");
            if ($serviceargs_serial != "") {
                $serviceargs = unserialize(base64_decode($serviceargs_serial));
            } else {
                $serviceargs = grab_array_var($inargs, "serviceargs");
            }

            $output = '
            <input type="hidden" name="address" value="' . encode_form_val($address) . '">
            <input type="hidden" name="hostnames" value="' . base64_encode(serialize($hostnames)) . '">
            <input type="hidden" name="bucketlist" value="' . encode_form_val($bucketlist) . '">
            <input type="hidden" name="accesskeyid" value="' . encode_form_val($accesskeyid) . '">
            <input type="hidden" name="accesskey" value="' . encode_form_val($accesskey) . '">
            <input type="hidden" name="region" value="' . encode_form_val($region) . '">
            <input type="hidden" name="staticcreds" value="' . encode_form_val($staticcreds) . '">
            <input type="hidden" name="staticconfig" value="' . encode_form_val($staticconfig) . '">
            <input type="hidden" name="credsfilepath" value="' . encode_form_val($credsfilepath) . '">
            <input type="hidden" name="configfilepath" value="' . encode_form_val($configfilepath) . '">
            <input type="hidden" name="services_serial" value="' . base64_encode(serialize($services)) . '">
            <input type="hidden" name="serviceargs_serial" value="' . base64_encode(serialize($serviceargs)) . '">';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            $check_interval = grab_array_var($inargs, "check_interval", "");
            $hostnames_serial = grab_array_var($inargs, "hostnames", "");

            $hostnames = unserialize(base64_decode($hostnames_serial));

            // Check for errors
            $errors = 0;
            $errmsg = array();
            if ($check_interval < 5) {
                $errmsg[$errors++] = "Check interval cannot be less than five minutes. This is because AWS sends CloudWatch data every five minutes. Querying between the time the last data was was received and now - if less than five minutes - will result in an empty response from CloudWatch.";
            }
            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }
            break;

            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:

            $output = '
            ';
            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostnames_serial = grab_array_var($inargs, "hostnames");
            $hostnames = unserialize(base64_decode($hostnames_serial));

            $bucketlist = grab_array_var($inargs, "bucketlist", "");
            $accesskeyid = grab_array_var($inargs, "accesskeyid", "");
            $accesskey = grab_array_var($inargs, "accesskey", "");
            $staticcreds = grab_array_var($inargs, "staticcreds", "");
            $staticconfig = grab_array_var($inargs, "staticconfig", "");
            $credsfilepath = grab_array_var($inargs, "credsfilepath", "");
            $configfilepath = grab_array_var($inargs, "configfilepath", "");
            $region = grab_array_var($inargs, "region", "");
            $check_interval = grab_array_var($inargs, "check_interval", "");

            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = unserialize(base64_decode($services_serial));
            $serviceargs = unserialize(base64_decode($serviceargs_serial));

            // Save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostnames"] = $hostnames;
            $meta_arr["address"] = $address;
            $meta_arr["bucketlist"] = $bucketlist;
            $meta_arr["accesskeyid"] = $accesskeyid;
            $meta_arr["accesskey"] = $accesskey;
            $meta_arr["staticcreds"] = $staticcreds;
            $meta_arr["staticconfig"] = $staticconfig;
            $meta_arr["credsfilepath"] = $credsfilepath;
            $meta_arr["configfilepath"] = $configfilepath;
            $meta_arr["region"] = $region;
            $meta_arr["services"] = $services;
            $meta_arr["serivceargs"] = $serviceargs;
            save_configwizard_object_meta($wizard_name, $hostnames, "", $meta_arr);

            $objs = array();

            // Append credentials file path if toggled, otherwise pass in bucketlist, accesskeyid and accesskey
            if ($staticcreds == "on") {
                $commonopts .= "--credfile '" . $credsfilepath . "' ";
            } else {
                $commonopts .= "--accesskeyid '" . $accesskeyid . "' --secretaccesskey '" . $accesskey . "' ";
            }

            // Append config file path if toggled, otherwise pass in region
            if ($staticconfig == "on") {
                $commonopts .= "--configfile '" . $configfilepath . "' ";
            }

            // Create non-existent hosts
            foreach ($serviceargs as $service => $args) {

                if (!host_exists($args["Hostname"])) {
                    $objs[] = array(
                        "type" => OBJECTTYPE_HOST,
                        "use" => "xiwizard_linuxserver_host",
                        "host_name" => $args["Hostname"],
                        "icon_image" => "s3.png",
                        "statusmap_image" => "s3.png",
                        "_xiwizard" => $wizard_name,
                        "check_command" => "check_s3!" . "--changemode 'checkalive' --bucketname '" . $service . "' " . $commonopts
                    );
                }

            }

            $service_name_dict = array(

                    "NumberOfObjects" => "Number of Objects",
                    "BucketSizeBytes" => "Bucket Size Bytes",
                    "AllRequests" => "All Requests",
                    "DeleteRequests" => "Delete Requests",
                    "GetRequests" => "Get Requests",
                    "HeadRequests" => "Head Requests",
                    "PutRequests" => "Put Requests",
                    "PostRequests" => "Post Requests",
                    "ListRequests" => "List Requests",
                    "BytesDownloaded" => "Bytes Downloaded",
                    "BytesUploaded" => "Bytes Uploaded",
                    "4xxErrors" => "4xx Errors",
                    "5xxErrors" => "5xx Errors",
                    "FirstByteLatency" => "First Byte Latency",
                    "TotalRequestLatency" => "Total Request Latentcy"

            );

            // See which services we should monitor
            foreach ($serviceargs as $svc => $metric) {

                foreach ($metric as $metricname => $values) {

                    $dimensionopts = "";
                    
                    if ($values[$metricname . "_storagetype"] != "") {
                        $dimensionopts .= "--storagetype '" . $values[$metricname . "_storagetype"] . "' ";
                    }

                    if ($values[$metricname . "_filterid"] != "") {
                        $dimensionopts .= "--filterid '" . $values[$metricname . "_filterid"] . "' ";
                    }

                    if ($svc != "") {
                        $dimensionopts .= "--bucketname '" . $svc . "' ";
                    }

                    if ($svc["Region"] != "" && $statticconfig != "on") {
                        $dimensionopts .= "--region '" . $metric["Region"] . "' ";
                    }

                    switch ($metricname) {

                    case "BucketSizeBytes":

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $metric["Hostname"],
                            "address" => $svc,
                            "service_description" => $metric["Hostname"] . " " . $service_name_dict[$metricname],
                            "use" => "xiwizard_linuxserver_ping_service",
                            "check_command" => "check_s3!" . "-P 1440" . " --metricname '" . $metricname . "' " . $commonopts . "--warning '" . $values[$metricname . "_warning"] . "' --critical '" . $values[$metricname . "_critical"] . "' " . $dimensionopts,
                            "_xiwizard" => $wizard_name
                        );
                        break;

                    case "NumberOfObjects":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $metric["Hostname"],
                            "address" => $svc,
                            "service_description" => $metric["Hostname"] . " " . $service_name_dict[$metricname],
                            "use" => "xiwizard_linuxserver_ping_service",
                            "check_command" => "check_s3!" . "-P 1440" . " --metricname '" . $metricname . "' " . $commonopts . "--warning '" . $values[$metricname . "_warning"] . "' --critical '" . $values[$metricname . "_critical"] . "' " . $dimensionopts,
                            "_xiwizard" => $wizard_name
                        );
                        break;

                    case "Region":
                        break;

                    case "Hostname":
                        break;

                    default:
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $metric["Hostname"],
                            "service_description" => $metric["Hostname"] . " " . $service_name_dict[$metricname],
                            "use" => "xiwizard_linuxserver_ping_service",
                            "check_command" => "check_s3!" . "-P " . $check_interval . " --metricname '" . $metricname . "' " . $commonopts . "--warning '" . $values[$metricname . "_warning"] . "' --critical '" . $values[$metricname . "_critical"] . "' " . $dimensionopts,
                            "_xiwizard" => $wizard_name
                        );
                        break;
                    }
                }
            }

            // Return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}


/**
 *
 * @return string
 */
